scmPlayernames = CreateFrame("Frame")
local scmPlayernames = scmPlayernames

SCM_PLAYERNAMES_LEFTBRACKET = ""
SCM_PLAYERNAMES_RIGHTBRACKET = ""
SCM_PLAYERNAMES_MOUSEOVER = nil

SCM_PLAYERNAMES_COLORS = {
	DEATHKNIGHT = "c41e3a",
	DRUID       = "ff7c0a",
	HUNTER      = "aad372",
	MAGE        = "68ccef",
	PALADIN     = "f48cba",
	PRIEST      = "ffffff",
	ROGUE       = "fff468",
	-- SHAMAN      = "00dbba", -- Old lightblue shaman color
	SHAMAN      = "2359ff",
	WARLOCK     = "9382c9",
	WARRIOR     = "c69b6d"
}

local classes = {} -- Key: Name, Value: Class
local names = setmetatable({}, {__index =
	function(self, key) -- Key: Name
		if classes[key] then
			self[key] = string.format("|cff%s%s|r", SCM_PLAYERNAMES_COLORS[classes[key]], key)
		else
			self[key] = string.format("|cffa0a0a0%s|r", key)
		end
		return self[key]
	end
})

local function addName(name, class)
	if not class or not name then return end
	names[name] = nil
	if classes[name] then return end
	if class == "UNKNOWN" or not SCM_PLAYERNAMES_COLORS[class] then return end
	classes[name] = class
end

local orig = nil
local function addMessage(frame, text, ...)
	local name = arg2
	if event == "CHAT_MSG_SYSTEM" then name = select(3, text:find("|h%[(.+)%]|h")) end
	if type(name) == "string" then text = text:gsub("|h%["..name.."%]|h", "|h"..SCM_PLAYERNAMES_LEFTBRACKET..names[name]..SCM_PLAYERNAMES_RIGHTBRACKET.."|h") end
	return orig(frame, text, ...)
end

scmPlayernames:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)

function scmPlayernames:Enable()
	orig = ChatFrame1.AddMessage
	ChatFrame1.AddMessage = addMessage

	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("WHO_LIST_UPDATE")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("CHAT_MSG_SYSTEM")
	self:RegisterEvent("FRIENDLIST_UPDATE")
	self:RegisterEvent("GUILD_ROSTER_UPDATE")

	if SCM_PLAYERNAMES_MOUSEOVER then
		self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	end

	local pclass = select(2, UnitClass("player"))
	addName(UnitName("player"), pclass)

	if IsInGuild() then GuildRoster() end
end

function scmPlayernames:Disable()
	ChatFrame1.AddMessage = orig
	self:UnregisterAllEvents()
end

function scmPlayernames:FRIENDLIST_UPDATE()
	for i = 1, GetNumFriends() do
		local name, _, class = GetFriendInfo(i)
		addName(name, class:upper())
	end
end

function scmPlayernames:GUILD_ROSTER_UPDATE()
	for i = 1, GetNumGuildMembers() do
		local name, _, _, _, _, _, _, _, _, _, class = GetGuildRosterInfo(i)
		addName(name, class)
	end
end

function scmPlayernames:RAID_ROSTER_UPDATE()
	for i = 1, GetNumRaidMembers() do
		local n = GetRaidRosterInfo(i)
		local _, c = UnitClass(n)
		addName(n, c)
	end
end

function scmPlayernames:PARTY_MEMBERS_CHANGED()
	for i = 1, GetNumPartyMembers() do
		local n = UnitName("party" .. i)
		local _, c = UnitClass("party" .. i)
		addName(n, c)
	end
end

function scmPlayernames:PLAYER_TARGET_CHANGED()
	if not UnitExists("target") or not UnitIsPlayer("target") or not UnitIsFriend("player", "target") then return end
	addName(UnitName("target"), select(2, UnitClass("target")))
end

function scmPlayernames:UPDATE_MOUSEOVER_UNIT()
	if not UnitExists("mouseover") or not UnitIsPlayer("mouseover") or not UnitIsFriend("player", "mouseover") then return end
	addName(UnitName("mouseover"), select(2, UnitClass("mouseover")))
end

function scmPlayernames:CHAT_MSG_SYSTEM(msg)
	local name, class = select(3, msg:find("^|Hplayer:%w+|h%[(%w+)%]|h: Level %d+ %w+ (%w+) %- .+$"))
	if name and class then
		addName(name, class:upper())
	end
end

function scmPlayernames:WHO_LIST_UPDATE()
	for i = 1, GetNumWhoResults() do
		local name, _, _, _, class = GetWhoInfo(i)
		addName(name, class:upper())
	end
end

scmPlayernames:Enable()

